@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Utils for file and directory operations.
 
 This provides enhanced and more error resilient forms of standard
@@ -28,6 +14,7 @@ import errno
 import fnmatch
 import glob
 import os
+import pickle
 import shutil
 import stat
 import sys
@@ -298,8 +285,20 @@ def getFilenameRealPath(path):
         doesn't handle file symlinks at the end on older Python currently, but
         we shouldn't deal with those.
     """
+    orig_path = path
     path = os.path.realpath(path)
 
+    # Avoid network mounts being converted to UNC shared paths ny newer
+    # Python versions, many tools won't work with those.
+    if os.name == "nt" and path.startswith("\\\\"):
+        drive, _remaining_path = os.path.splitdrive(orig_path)
+
+        if drive:
+            drive_real_path = os.path.realpath(drive + "\\")
+            assert path.startswith(drive_real_path)
+
+            path = drive + path[len(drive_real_path) :]
+
     # Attempt to resolve Windows symlinks older Python
     if os.name == "nt":
         path = path.strip(os.path.sep)
@@ -317,7 +316,7 @@ def getFilenameRealPath(path):
                 if os.path.sep not in dirname:
                     dirname = dirname + os.path.sep
 
-                return os.path.normpath(os.path.join(dirname, filename))
+                path = os.path.normpath(os.path.join(dirname, filename))
 
     return path
 
@@ -819,6 +818,22 @@ def putTextFileContents(filename, contents, encoding=None):
             _writeContents(output_file)
 
 
+def putBinaryFileContents(filename, contents):
+    """Write a binary file from given contents.
+
+    Args:
+        filename: str with the file to be created
+        contents: bytes that should be written into the file
+
+    Returns:
+        None
+    """
+
+    with withFileLock("writing file %s" % filename):
+        with openTextFile(filename, "wb") as output_file:
+            output_file.write(contents)
+
+
 def changeTextFileContents(filename, contents, encoding=None, compare_only=False):
     """Write a text file from given contents.
 
@@ -1173,11 +1188,11 @@ def getExternalUsePath(filename, only_dirname=False):
     filename = os.path.abspath(filename)
 
     if os.name == "nt":
+        key = filename, only_dirname
+
         if filename not in _external_use_path_cache:
             filename = getFilenameRealPath(filename)
 
-            asked_filename = filename
-
             if only_dirname:
                 dirname = getWindowsShortPathName(os.path.dirname(filename))
                 assert os.path.exists(dirname)
@@ -1185,10 +1200,13 @@ def getExternalUsePath(filename, only_dirname=False):
             else:
                 filename = getWindowsShortPathName(filename)
 
-            _external_use_path_cache[asked_filename] = filename
-            _external_use_path_cache[filename] = filename
+            _external_use_path_cache[key] = filename
 
-        return _external_use_path_cache[filename]
+            # Looking up again should give same result immediately.
+            key = filename, only_dirname
+            _external_use_path_cache[key] = filename
+
+        return _external_use_path_cache[key]
     else:
         return filename
 
@@ -1403,3 +1421,30 @@ def isFilesystemEncodable(filename):
         return value == filename
     else:
         return True
+
+
+def openPickleFile(filename, mode, protocol=-1):
+    file_handle = openTextFile(filename, mode)
+
+    if python_version < 0x300:
+        return file_handle, pickle.Pickler(file_handle, protocol)
+    else:
+        return file_handle, pickle._Pickler(  # pylint: disable=protected-access
+            file_handle, protocol
+        )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
